package comunicacao;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;

import comunicacao.data.MainBuffer;
import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

public class Server {
	private byte[] sendData;
	private byte[] receiveData;
	private int windowSize;
	private SequenceNumber windowBase;	
	private SequenceNumber currentSeqNumber;
	private int nonAckedPackets;
	private Segment[] fileParts;

	private static DatagramSocket serverSocket;
	public static final int SERVER_PORT = 1748;
	public static final long TIME_OUT = 500;



	private Server(byte[] sendData, byte[] receiveData,
			DatagramSocket serverSocket, int windowSize,
			SequenceNumber windowBase, SequenceNumber currentSeqNumber,
			int nonAckedPackets, Segment[] fileParts) {
		super();
		this.sendData = sendData;
		this.receiveData = receiveData;
		this.serverSocket = serverSocket;
		this.windowSize = windowSize;
		this.windowBase = windowBase;
		this.currentSeqNumber = currentSeqNumber;
		this.nonAckedPackets = nonAckedPackets;
		this.fileParts = fileParts;	
	}

	
	
	
	

	public byte[] getSendData() {
		return sendData;
	}


	public void setSendData(byte[] sendData) {
		this.sendData = sendData;
	}


	public byte[] getReceiveData() {
		return receiveData;
	}


	public void setReceiveData(byte[] receiveData) {
		this.receiveData = receiveData;
	}


	public static synchronized DatagramSocket getServerSocket() throws SocketException {
		if (Server.serverSocket == null){
			Server.serverSocket = new DatagramSocket(SERVER_PORT);
		}

		return Server.serverSocket;
	}


	public void setServerSocket(DatagramSocket serverSocket) {

	}


	public int getWindowSize() {
		return windowSize;
	}


	public void setWindowSize(int windowSize) {
		this.windowSize = windowSize;
	}


	public SequenceNumber getWindowBase() {
		return windowBase;
	}


	public void setWindowBase(SequenceNumber windowBase) {
		this.windowBase = windowBase;
	}


	public SequenceNumber getCurrentSeqNumber() {
		return currentSeqNumber;
	}


	public void setCurrentSeqNumber(SequenceNumber currentSeqNumber) {
		this.currentSeqNumber = currentSeqNumber;
	}


	public int getNonAckedPackets() {
		return nonAckedPackets;
	}

	public void setNonAckedPackets(int nonAckedPackets) {
		this.nonAckedPackets = nonAckedPackets;
	}

	public Segment[] getFileParts() {
		return fileParts;
	}

	public void setFileParts(Segment[] fileParts) {
		this.fileParts = fileParts;
	}

	public void sendSynAck(InetAddress requesterAdress, int resquesterPort, SequenceNumber ackNum)
			throws IOException, HeaderDontFitException {

		Header header = new Header(new HeaderFlags(true, false, false, true, false, false, 0), ackNum);
		header.putHeader(sendData);

		DatagramPacket packet = new DatagramPacket(sendData, Header.HEADER_SIZE,
				requesterAdress, resquesterPort);
		serverSocket.send(packet);
	}

	public void ackReceived(InetAddress clientIP, int clientPort){


	}

	public DatagramPacket waitForIt() throws SocketTimeoutException,
	IOException {
		DatagramPacket packet = new DatagramPacket(receiveData,
				receiveData.length);

		try {
			serverSocket.receive(packet);
		} catch (SocketTimeoutException e) {
			throw e;
		}
		return packet;
	}

	public boolean openConnection(InetAddress clientIP, int clientPort, SequenceNumber ackNum) throws IOException, HeaderDontFitException{
		this.sendSynAck(clientIP, clientPort, ackNum);

		// Wait for ACK
		this.serverSocket.setSoTimeout(3000);
		DatagramPacket receivePacket = null;
		
		try {
			receivePacket = this.waitForIt();
		} catch (SocketTimeoutException e) {
			return false;
		}
		
		if(receivePacket != null){
			Header header = new Header(receivePacket);
			if(header.getFlags().isAck()){
				//Connection established, start sending data.
				return true;
			}
		}
		
		return false;

	}

//	public void sendData(Data data, DatagramSocket socket) throws IOException {
//		DatagramPacket packet = new DatagramPacket(data.getByteBuffer(),
//				data.getByteBuffer().length, socket.getInetAddress(),
//				socket.getPort());
//		socket.send(packet);
//	}






	public static MainBuffer requestBuffer(int i) {
		// TODO Reuse buffers if possible
		byte[] buffer = new byte[i];
		MainBuffer returned = new MainBuffer(buffer, 0, buffer.length);				
		return returned;
	}



	//	public static void main(String[] args) {
	//
	//		try {
	//			byte[] sendData = new byte[1024];
	//			byte[] receiveData = new byte[1024];
	//			DatagramSocket serverSocket = new DatagramSocket(1748);
	//			// Server
	//			Servidor servidor = new Servidor(sendData, receiveData, serverSocket, , , , , );
	//			System.out.println("oi");
	//			DatagramPacket receivedPacket = servidor.waitForIt();
	//			System.out.println("oi");
	//			HeaderFlags header = HeaderFlags
	//					.getHeader(receivedPacket.getData()[0]);
	//
	//			if (header.equals(HeaderFlags.SYN)){
	//				System.out.println("Chegou um syn");
	//				//Sending synack
	//				servidor.sendSynAck(receivedPacket.getAddress(), receivedPacket.getPort());
	//
	//				//Wait for ack
	//				receivedPacket = servidor.waitForIt();
	//
	//				header = HeaderFlags
	//						.getHeader(receivedPacket.getData()[0]);
	//				if (header.equals(HeaderFlags.ACK)){
	//					//Ack received, send file
	//					servidor.ackReceived(receivedPacket.getAddress(), receivedPacket.getPort());
	//
	//
	//				}
	//
	//			} 
	//
	//
	//		} catch (SocketException e) {
	//			// TODO Auto-generated catch block
	//			e.printStackTrace();
	//		} catch (IOException e) {
	//			// TODO Auto-generated catch block
	//			e.printStackTrace();
	//		}
	//
	//	}



}
