package comunicacao;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.HashMap;

import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

/**
 * Class that simulates a TCP ServerSocket using UDP DatagramsSockets
 * 
 * @author jpmc
 * 
 */
public class ServerSocket implements IServerSocket {

	private DatagramSocket socket;// Socket used for receiving connection
	// establishment requests
	private int port;// Port alocated for this server

	private byte[] receivingBuffer;// Buffer for receiving requests
	private byte[] sendingBuffer;// Buffer for receiving response

	private HashMap<InetAddress, Socket> connectionsOpened;

	public ServerSocket(int port) throws SocketException {
		this.socket = new DatagramSocket(port);
		this.port = port;
		this.receivingBuffer = new byte[1024];
		this.sendingBuffer = new byte[1024];

		this.connectionsOpened = new HashMap<InetAddress, Socket>();
	}

	@Override
	public ISocket accept() throws IOException {
		// TODO Auto-generated method stub
		DatagramPacket receivePacket = new DatagramPacket(receivingBuffer,
				receivingBuffer.length);
		DatagramPacket responsePacket;
		Header receiveHeader, responseHeader;
		Socket connection = null;

		/*------------------------------WAITING FOR SYN----------------------------*/
		System.out.println("Server socket is waiting for a syn");
		this.socket.receive(receivePacket);

		receiveHeader = new Header(receivePacket);
		if (receiveHeader.getFlags().isSyn()) {
			System.out.println("Server socket confirmed it was a syn");
			connection = this.connectionsOpened.get(receivePacket.getAddress());
			if (connection == null) {

				connection = new Socket(receivePacket.getAddress(),
						receivePacket.getPort());
				this.connectionsOpened.put(receivePacket.getAddress(),
						connection);

			}
			/*------------------SENDING SYNACK--------------------------*/
			// Send SYNACK
			responseHeader = new Header(new HeaderFlags(true, false, false,
					true, false, false, 0), new SequenceNumber(connection.getPort()));
		
			try {
				responseHeader.putHeader(sendingBuffer);
			} catch (HeaderDontFitException e) {
				// Won't happen
				e.printStackTrace();
			}

			responsePacket = new DatagramPacket(sendingBuffer,
					Header.HEADER_SIZE, receivePacket.getAddress(),
					receivePacket.getPort());
			System.out.println("Server socket is sending a synack");
			this.socket.send(responsePacket);

			/*------------------WAIT FOR ACK-----------------*/
			// TODO wait for ack
			// TODO set timeout for waiting for an ack, try it (resending the
			// synack) a few times before giving up, if giveUp, remeber to close
			// the openned socket(kill the thread and remove it from the
			// mapping)

			boolean ackReceived = false;

			
			connection.getDatagramSocket().setSoTimeout(2000);
			for (int i = 0; i < 5 && !ackReceived; i++) {
				
				try {
					System.out.println("Server socket is waiting for an ack");
					connection.getDatagramSocket().receive(receivePacket);
					System.out.println("Server socket received an ack");
					ackReceived = true;
				} catch (SocketTimeoutException e) {
					System.out.println("Server socket is resending the synack");
					this.socket.send(responsePacket);
				}
			}
			if (ackReceived) {
				Header receivedHeader = new Header(receivePacket);
				if (receivedHeader.getFlags().isAck()) {
					connection.getDatagramSocket().setSoTimeout(0);
					return connection;
				} else {
					Header sendingHeader = new Header(new HeaderFlags(false, false, true, false, false, false, 0), new SequenceNumber(0));
					try {
						sendingHeader.putHeader(sendingBuffer);
					} catch (HeaderDontFitException e) {
						e.printStackTrace();
					}
					return null;
				}
			} else {
				Header sendingHeader = new Header(new HeaderFlags(false, false, true, false, false, false, 0), new SequenceNumber(0));
				try {
					sendingHeader.putHeader(sendingBuffer);
				} catch (HeaderDontFitException e) {
					e.printStackTrace();
				}
				return null;
			}

		} else {
			Header sendingHeader = new Header(new HeaderFlags(false, false, true, false, false, false, 0), new SequenceNumber(0));
			System.out.println("Server is sending a rst");
			try {
				sendingHeader.putHeader(sendingBuffer);
			} catch (HeaderDontFitException e) {
				e.printStackTrace();
			}
			this.socket.send(new DatagramPacket(sendingBuffer, Header.HEADER_SIZE, receivePacket.getAddress(), receivePacket.getPort()));
			return null;
			
		}
	}

//	@Override
//	public void close() throws IOException {
//		Socket connection = null;
//
//		/*-----------------------------RECEIVING FYN----------------------------*/
//		DatagramPacket receivedFyn = new DatagramPacket(receivingBuffer,
//				receivingBuffer.length);
//
//		this.socket.receive(receivedFyn);
//
//		Header receiveHeader = new Header(receivedFyn);
//		if (receiveHeader.getFlags().isFyn()) {
//			connection = this.connectionsOpened.get(receivedFyn.getAddress());
//		}
//
//		/*--------------------------SENDING FYNACK------------------------*/
//		DatagramPacket fynack = new DatagramPacket(sendingBuffer,
//				Header.HEADER_SIZE, receivedFyn.getAddress(),
//				receivedFyn.getPort());
//
//		Header sendingHeader = new Header(new HeaderFlags(false, true, false,
//				true, false, false, 0), new SequenceNumber(0));
//		try {
//			sendingHeader.putHeader(sendingBuffer);
//		} catch (HeaderDontFitException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//
//		this.socket.send(fynack);
//		// TODO Auto-generated method stub
//
//		/*--------------------------WAITING ACK---------------------------*/
//		boolean receivedAck = false;
//
//		this.socket.setSoTimeout(2000);
//		DatagramPacket ack = new DatagramPacket(receivingBuffer,
//				receivingBuffer.length);
//		for (int i = 0; i < 5 && !receivedAck; i++) {
//			try {
//				this.socket.receive(ack);
//				receivedAck = true;
//				this.connectionsOpened.remove(connection);
//			} catch (SocketTimeoutException e) {
//				this.socket.send(fynack);
//				e.printStackTrace();
//			}
//		}
//
//	}



	@Override
	public int getLocalPort() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public InetAddress getLocalAddress() {
		// TODO Auto-generated method stub
		return null;
	}

}
