package comunicacao.data;

import java.net.DatagramPacket;
import java.net.InetAddress;

import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

public class SegmentsConstructor {
	private byte[] originalData;
	private InetAddress remoteIp;
	private int remotePort;

	private byte[] atributeArray;
	private Segment[] segmentsArray;
	private SequenceNumber[] seqNumArray;
	private int[] copiedDataArray;

	private long seqNum;
	private int originalDataSize;
	private int totalOfSegments; // Number of segments
	private int offset; // Offset used for putting the Header in its respective
	// place
	private int copiedData; // Data that is been copied at the moment
	private int totalCopiedData; // Data that has been already copied

	public SegmentsConstructor(byte[] originalData, int originalDataSize,
			InetAddress remoteIP, int remotePort) throws HeaderDontFitException {
		this.originalData = originalData;
		this.remoteIp = remoteIP;
		this.remotePort = remotePort;
		this.originalDataSize = originalDataSize;

		this.atributeArray = new byte[10485760];
		this.segmentsArray = new Segment[18000];
		this.seqNumArray = new SequenceNumber[18000]; // Array that is going to
		// keep the sequence
		// numbers for later use
		this.copiedDataArray = new int[18000];

		this.totalOfSegments = 0;
		this.seqNum = 0;
		this.offset = 0; // offset used for "putting" the header in its
		// respective place inside the
		// byte[] "atributeArray"
		this.copiedData = 0;
		this.totalCopiedData = 0;

		Header header;
		int seqNumIndex = 0;
		int arrayIndex = Header.HEADER_SIZE;

		for (int i = 0; i < originalDataSize; i++) {

			/*-----------------------COPYING DATA------------------------*/

			this.atributeArray[arrayIndex + i] = this.originalData[i];
			this.copiedData++;
			this.totalCopiedData++;

			if ((this.copiedData == Segment.MAXIMUM_DATA_SIZE)
					|| (this.totalCopiedData == this.originalDataSize)) {

				this.seqNumArray[seqNumIndex] = new SequenceNumber(this.seqNum);
				this.seqNum += this.copiedData;

				/*--------------------ATTACHING HEADER---------------------*/

				header = new Header(new HeaderFlags(false, false, false, false,
						false, false, 0), this.seqNumArray[seqNumIndex]);
				header.putHeader(this.atributeArray, this.offset);
				System.out.println("offset: " + this.offset);
				if (this.totalCopiedData != this.originalDataSize) {
					this.offset += Header.HEADER_SIZE + this.copiedData;
				}

				this.copiedDataArray[seqNumIndex] = this.copiedData;
				
				arrayIndex += Header.HEADER_SIZE;
				seqNumIndex++;
				this.copiedData = 0;

			}

		}

		/*----------------------------CREATING SEGMENTS-----------------------*/
		int index = 0;

		for (int i = 0; i < this.seqNumArray.length
				&& this.seqNumArray[i] != null; i++) {
			Segment segment = new Segment(new DatagramPacket(
					this.atributeArray, index, this.copiedDataArray[i]
							+ Header.HEADER_SIZE, this.remoteIp,
					this.remotePort), this.seqNumArray[i]);
			this.totalOfSegments++;

			this.segmentsArray[i] = segment;

			index += this.copiedDataArray[i] + Header.HEADER_SIZE;

		}

	}

	public Segment getSegment(int index) {
		return this.segmentsArray[index];
	}

	public Segment getSegment(SequenceNumber seqNum) {
		for (int i = 0; i < this.segmentsArray.length; i++) {
			if (this.segmentsArray[i] != null
					&& this.segmentsArray[i].getSequenceNumber().equals(seqNum)) {
				return this.segmentsArray[i];
			}
		}
		return null;
	}

	public int getNumberOfSegments() {
		return totalOfSegments;
	}

}
