package comunicacao.data.segment;

import java.io.Serializable;

import comunicacao.util.UnsignedInteger;

public class SequenceNumber extends UnsignedInteger implements
		Comparable<SequenceNumber> {
	private byte leftByte;
	private byte middleLeftByte;
	private byte middleRightByte;
	private byte rightByte;

	public SequenceNumber(long unsignedNumber) {
		super(unsignedNumber);
		this.intToByte(this.getSignedValue());
	}

	public SequenceNumber(byte leftByte, byte middleLeftByte,
			byte middleRightByte, byte rightByte) {
		super(0);
		this.leftByte = leftByte;
		this.middleLeftByte = middleLeftByte;
		this.middleRightByte = middleRightByte;
		this.rightByte = rightByte;

		int number = this.leftByte;
		number = (number << 8) + (this.middleLeftByte & 0xff);
		number = (number << 8) + (this.middleRightByte & 0xff);
		number = (number << 8) + (this.rightByte & 0xff);
		this.directlySetValue(number);

	}

	public byte getLeftByte() {
		return leftByte;
	}

	public void setLeftByte(byte leftByte) {
		this.leftByte = leftByte;
	}

	public byte getMiddleLeftByte() {
		return middleLeftByte;
	}

	public void setMiddleLeftByte(byte middleLeftByte) {
		this.middleLeftByte = middleLeftByte;
	}

	public byte getMiddleRightByte() {
		return middleRightByte;
	}

	public void setMiddleRightByte(byte middleRightByte) {
		this.middleRightByte = middleRightByte;
	}

	public byte getRightByte() {
		return rightByte;
	}

	public void setRightByte(byte rightByte) {
		this.rightByte = rightByte;
	}

	public void add(int inc) {
		super.add(inc);
		this.intToByte(this.getSignedValue());

	}

	private void intToByte(long number) {

		this.rightByte = (byte) (number & 0xff);
		this.middleRightByte = (byte) ((number >> 8) & 0xff);
		this.middleLeftByte = (byte) ((number >> 16) & 0xff);
		this.leftByte = (byte) ((number >> 24) & 0xff);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + leftByte;
		result = prime * result + middleLeftByte;
		result = prime * result + middleRightByte;
		result = prime * result + rightByte;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		SequenceNumber other = (SequenceNumber) obj;
		if (leftByte != other.leftByte)
			return false;
		if (middleLeftByte != other.middleLeftByte)
			return false;
		if (middleRightByte != other.middleRightByte)
			return false;
		if (rightByte != other.rightByte)
			return false;
		return true;
	}

	@Override
	public int compareTo(SequenceNumber o) {
		return this.getSignedValue() - o.getSignedValue();
	}

}
