package comunicacao.sender;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import comunicacao.data.WindowClient;
import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

public class ReceiverThread implements Runnable {
	private DatagramSocket socket;
	private byte[] receiveBuffer;

	private byte[] receivePacketBuffer;
	private DatagramPacket receivePacket;

	private byte[] ackSenderBuffer;
	private DatagramPacket ackSenderPacket;

	private WindowClient window;
	
	private int totalAmountOfData;
	private int amountOfDataReceived;

	public ReceiverThread(DatagramSocket socket, byte[] receiveBuffer, int length,
			InetAddress remoteAddress, int remotePort) {
		this.socket = socket;
		this.receiveBuffer = receiveBuffer;

		this.receivePacketBuffer = new byte[1024];
		this.receivePacket = new DatagramPacket(this.receivePacketBuffer,
				this.receivePacketBuffer.length);

		this.ackSenderBuffer = new byte[1024];
		this.ackSenderPacket = new DatagramPacket(this.ackSenderBuffer,
				Header.HEADER_SIZE, remoteAddress, remotePort);

		this.window = new WindowClient(new SequenceNumber(0),
				this.receiveBuffer.length);
		
		this.totalAmountOfData = length;
		this.amountOfDataReceived = 0;
	}

	public void run() {
		
		System.out.println("Receiver initialized");
		while (!hasFinished()){
			try {
				this.socket.receive(this.receivePacket);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			this.saveData(this.receivePacket);
			
			Header header = new Header(this.receivePacket);
			
			Header sendHeader = new Header(new HeaderFlags(false, false, false, true, false, false, 0), header.getSeqNum());
			try {
				sendHeader.putHeader(this.ackSenderBuffer);
			} catch (HeaderDontFitException e) {
				//Won't happen
				e.printStackTrace();
			}
			
			try {
				System.out.println("Receiver is sending ack " + header.getSeqNum().value());
				this.socket.send(this.ackSenderPacket);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
			
		}
		

	}

	public void saveData(DatagramPacket packet) {

		Header packetHeader = new Header(packet);
		System.out.println("Receiver receved packet " + packetHeader.getSeqNum().value());
		if (!this.window.isDuplicated(packetHeader.getSeqNum())) {
			int length = packet.getLength() - Header.HEADER_SIZE;
			int index = (int) packetHeader.getSeqNum().value();
			System.out.println("index: " + index);
			for (int i = 0, j = Header.HEADER_SIZE; i < length; i++, j++) {
				this.receiveBuffer[index + i] = packet.getData()[j];
			}
			this.amountOfDataReceived += length;
			System.out.println("Receiver saved " + length + " bytes");
			this.window.packetReceived(packetHeader.getSeqNum(), length);
		}
		
		
		
	}

	
	public boolean hasFinished(){
		return this.totalAmountOfData <= this.amountOfDataReceived;
	}
	
	public DatagramSocket getSocket() {
		return this.socket;
	}

	public void setSocket(DatagramSocket socket) {
		this.socket = socket;
	}

	public byte[] getReceiveBuffer() {
		return receiveBuffer;
	}

	public void setReceiveBuffer(byte[] receiveBuffer) {
		this.receiveBuffer = receiveBuffer;
	}

}